
# q-expansion of newform 368.2.m.c, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 368
weight = 2
poly_data = [1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[47, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [277, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [97, [0, 0, 0, 0, 1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 1, -1, 0, 0, 0, -1], [-1, 1, -2, 1, -1, 0, -1, 0, 0, 1], [0, 1, -1, 0, -1, -1, -1, 0, -1, 1], [-1, 1, -1, 1, 0, 1, 0, -2, 0, 1], [1, -1, 0, -2, 1, -2, 2, -2, 1, -2], [-1, -1, 1, 3, 1, -1, -1, 0, 0, 0], [2, -2, 4, -4, 2, -2, 0, -2, 4, -2], [2, 1, 1, 1, 0, -3, 3, 0, 1, -2], [2, 2, 1, -2, 1, 2, 2, 0, 2, -2], [-1, 1, -1, 0, 1, 1, 1, -1, -1, -1], [-4, 2, -3, 3, -3, 3, -2, 4, 0, 1], [1, 2, 2, 2, 1, 0, 1, -1, 1, -1], [1, 0, 1, 0, -2, 2, 0, -1, 0, -1], [0, 0, -1, 3, -2, 3, -3, 2, -3, 1], [0, 2, -7, 1, -3, 3, -3, 1, -7, 2], [5, -5, 0, -3, 3, -3, 6, -3, 3, -3], [-1, 2, -1, 0, -3, 4, 2, -2, -4, 3], [-4, -3, 1, -2, -3, -2, 1, -3, -4, 0], [2, 1, 2, 0, 2, 0, 2, 1, 2, 0], [1, 1, 1, -1, -1, -1, 0, 3, -4, 3], [-3, 3, 0, 6, -6, 8, -2, 8, -6, 6], [-3, 4, -1, 0, 0, 1, -4, 3, 0, -1], [5, -4, 8, 3, 1, -1, -3, -8, 4, -5], [-2, -3, 4, -3, -2, 0, 5, 2, -2, -5], [-4, -4, -2, -3, 3, 2, 4, 4, 0, 5], [5, -3, 3, -5, 0, -2, 10, -6, 10, -2], [-3, -1, -3, 0, 1, 5, 0, 0, -5, -1], [0, 1, -3, -5, -3, 1, 0, 0, 7, -7], [2, -1, 0, 0, 1, 0, 0, -1, 2, 0], [-2, 0, 0, 2, 0, -5, 6, 5, 6, -5], [0, 4, -3, 6, -6, -3, -6, 6, -3, 4], [-1, 0, -6, 0, -2, 4, -4, 2, 0, 6], [6, 0, 8, -9, 14, -7, 7, -14, 9, -8], [-4, 7, -7, 4, 0, 0, 0, 6, 0, 0], [-1, 1, 0, -8, 1, -5, 1, -5, 1, -8], [10, -5, 6, -6, 5, -10, 0, -8, 8, -8], [-10, 8, -11, 7, -12, 7, -11, 8, -10, 0], [0, 5, -1, 2, -1, 5, 0, 0, -3, 3], [1, 5, -5, -1, 0, -4, 0, 3, 0, -4], [1, 3, -6, 3, 1, 0, -4, -6, 6, 4], [-7, -5, 4, -4, 3, -3, 4, -4, 5, 7], [0, 10, -7, 2, -5, -6, -5, 2, -7, 10], [-1, 6, 3, -1, 1, -3, -6, 1, 0, -1], [8, -8, 0, -2, 3, -6, 1, -6, 3, -2], [5, -1, 5, 0, 0, 2, 0, 0, -2, 0], [-1, -1, 6, -6, 1, 1, 0, 2, 4, 2], [0, -12, 0, -6, 9, -8, 9, -6, 0, -12], [-7, -5, -5, 10, -11, 11, -10, 5, 5, 7], [1, 0, 4, 6, -1, -2, 2, 1, -6, -4], [2, -8, -11, 1, -5, 5, -1, 11, 8, -2], [-7, 2, 3, -2, 2, -3, -2, 7, 0, -7], [12, 1, -1, -12, 0, -2, 8, -7, 8, -2], [7, -6, 10, -5, 24, -5, 10, -6, 7, 0], [-9, -1, -1, 1, 1, 9, 0, -5, 8, -5], [7, -7, 0, -2, 7, 1, 15, 1, 7, -2], [0, -10, 11, -6, 7, -16, 7, -6, 11, -10], [-14, 4, 1, 4, -14, 0, -11, 14, -14, 11], [-12, 0, 2, -2, 2, 7, -7, -2, 2, -2], [-20, 9, -10, 9, -20, 0, -10, 9, -9, 10], [0, -1, 5, -12, 7, -5, 7, -12, 5, -1], [-2, 10, 10, -3, 10, 10, -2, 0, -4, 4], [-7, -2, -7, 0, 0, 11, -8, 8, -11, 0], [-4, 5, -5, 4, 0, -2, 4, -6, 4, -2], [10, -11, 9, 2, -2, -9, 11, -10, 0, -19], [8, 1, 7, 1, 8, 0, 4, 2, -2, -4], [-3, -1, -7, 12, -12, 7, 1, 3, 0, 14], [-2, 5, -2, 0, -7, 12, -9, 9, -12, 7], [6, 3, -5, 1, -3, 1, -5, 3, 6, 0], [-3, -11, -2, 2, 11, 3, 0, -1, 10, -1], [-2, -8, -2, 0, 4, 2, 15, -15, -2, -4], [5, -3, -2, 1, -1, 2, 3, -5, 0, -13], [7, 0, 11, -4, 6, 7, -7, -6, 4, -11], [-1, 8, 6, 8, -1, 0, -1, -8, 8, 1], [11, -2, 2, -11, 0, 4, 5, 4, 5, 4], [8, -13, 8, 0, 7, 1, -4, 4, -1, -7], [10, -16, 10, -9, 9, -9, 10, -16, 10, 0], [6, -16, 8, -1, 8, -16, 6, 0, 7, -7], [-6, 6, 0, -2, -4, 14, -1, 14, -4, -2], [3, -3, -3, -3, 3, 0, -13, 5, -5, 13], [-17, 15, 0, 15, -17, 0, -3, 1, -1, 3], [0, 8, 0, 5, 8, -16, 8, 5, 0, 8], [-8, 13, -7, -9, -7, 13, -8, 0, -8, 8], [-13, 11, -5, 5, -11, 13, 0, 12, 3, 12], [14, -12, 12, -10, 16, -10, 12, -12, 14, 0], [9, -16, 14, -22, 14, -16, 9, 0, 11, -11], [2, -11, 11, -2, 0, 0, 5, -8, 5, 0], [-11, 13, -24, 21, -13, 13, -21, 24, -13, 11], [-4, 0, -10, 7, 7, 7, -7, -7, -7, 10], [2, 2, -8, 17, 2, -2, -17, 8, -2, -2], [0, -5, -1, 4, 14, 0, 14, 4, -1, -5], [-7, -2, -3, 3, 2, 7, 0, -2, 16, -2], [9, -21, 7, -7, 21, -9, 0, -14, 6, -14], [-8, -7, -8, 0, -12, 3, 5, -5, -3, 12], [0, 4, 4, 17, -5, 3, -5, 17, 4, 4], [-16, 7, -3, 6, -6, 6, -6, 3, -7, 16], [1, 8, -15, 9, -1, 1, -9, 15, -8, -1], [-13, -3, -13, 0, 1, -8, -4, 4, 8, -1], [-11, 19, -18, 26, -18, 19, -11, 0, -17, 17], [-9, 7, -7, 9, 0, 0, -1, -11, -1, 0], [-3, 12, -21, 12, -3, 0, -14, 4, -4, 14], [12, -6, 7, -6, 12, 0, 8, 0, 0, -8], [18, -1, 1, -18, 0, -9, 11, -6, 11, -9], [4, -17, -4, -2, -4, -17, 4, 0, -6, 6], [9, -10, -2, 2, 10, -9, 0, 10, 6, 10], [-5, 5, -17, 11, 3, 11, -17, 5, -5, 0], [-12, 19, -19, 12, 0, 13, -23, 1, -23, 13], [-7, 5, -21, 5, -7, 0, -13, 8, -8, 13], [-7, 0, 10, -1, -6, -2, 2, 6, 1, -10], [1, 5, 8, 0, -1, 1, 0, -8, -5, -1], [5, 4, -7, 2, -2, 7, -4, -5, 0, 3], [-15, 11, -15, 0, -7, 26, -18, 18, -26, 7], [5, 4, -2, 2, -4, -5, 0, 5, 19, 5], [-8, -3, -6, 13, -4, 13, -6, -3, -8, 0], [-22, 22, 0, 19, -4, 19, -23, 19, -4, 19], [-7, -6, 5, -9, 10, -10, 9, -5, 6, 7], [1, 0, 1, 12, 11, -2, 2, -11, -12, -1], [0, 1, -8, -7, 18, -18, 7, 8, -1, 0], [8, -1, 16, 5, -5, -16, 1, -8, 0, 4], [-2, -9, -2, 0, -13, 4, 4, -4, -4, 13], [-20, 23, -20, 15, -20, 23, -20, 0, -16, 16], [13, -3, -4, 17, -4, -3, 13, 0, 9, -9], [7, -7, 0, -15, 7, 7, 7, 7, 7, -15], [0, -10, 7, -14, 11, 9, 11, -14, 7, -10], [3, 0, 10, 8, 3, -10, 10, -3, -8, -10], [6, 6, -6, -6, 0, 0, -4, 9, -4, 0], [2, -11, 1, -1, 11, -2, 0, -4, 9, -4], [5, -3, -14, -7, -14, -3, 5, 0, -5, 5], [-1, 6, -23, 9, -9, 23, -6, 1, 0, -1], [-1, -4, -3, -4, -15, 15, 4, 3, 4, 1], [9, 7, 4, 0, -4, 4, 0, -4, -7, -9], [0, -7, 14, 13, -10, -2, -10, 13, 14, -7], [-16, -8, -16, 0, 7, 8, -7, 7, -8, -7], [-7, -7, -12, 5, 0, 5, -12, -7, -7, 0], [13, -2, 22, -8, 3, -8, 22, -2, 13, 0], [-18, 18, 0, 27, -19, 28, -31, 28, -19, 27], [3, -3, -8, 13, -13, 8, 3, -3, 0, 5], [17, -5, 8, -5, 17, 0, 3, 3, -3, -3], [0, -5, 0, 0, 12, 3, 23, -23, -3, -12], [-6, -1, 6, -6, 1, 6, 0, -13, -8, -13], [-7, 19, -1, 35, -1, 19, -7, 0, -10, 10], [0, 2, -17, 1, -16, 7, -16, 1, -17, 2], [9, -5, 6, -5, 9, 0, 21, 5, -5, -21], [-5, 0, -1, -17, -16, 2, -2, 16, 17, 1], [-7, 0, 6, -5, -1, -13, 13, 1, 5, -6], [-14, 23, -23, 14, 0, 5, -21, -7, -21, 5], [0, -22, 6, -8, 5, -8, 6, -22, 0, 0], [10, 1, 4, 1, 4, 1, 10, 0, -21, 21], [-6, -13, -6, 0, -7, -7, -17, 17, 7, 7], [9, -6, 6, -9, 0, -13, 5, 7, 5, -13], [3, 4, 4, 2, 1, -1, -2, -4, -4, -3], [0, -9, 15, -4, 6, 4, 6, -4, 15, -9], [20, -20, 19, -11, 11, -19, 20, -20, 0, -21], [-18, 18, 0, 1, -18, 0, -16, 0, -18, 1], [0, 0, 0, -1, -15, -6, 0, -6, -15, -1], [19, -9, 4, -8, 8, -4, 9, -19, 0, -29], [-4, 0, -33, 16, -20, 18, -18, 20, -16, 33], [5, 7, -4, 4, -4, 4, -7, -5, 0, 19], [13, 6, 3, 2, 3, 6, 13, 0, -4, 4], [-5, 22, 5, 9, -18, 9, 5, 22, -5, 0], [18, -6, -2, 2, 6, -18, 0, -20, 17, -20], [8, -8, 0, -20, 17, -22, 11, -22, 17, -20], [-8, 0, -5, -10, -8, 9, -9, 8, 10, 5], [-16, 23, -9, 23, -16, 0, -6, 32, -32, 6], [-3, 3, -3, 3, 0, -24, 6, -18, 6, -24], [-23, 0, 7, 0, 7, 0, -23, 0, 5, -5], [5, 10, 4, -28, 10, -28, 4, 10, 5, 0], [-11, 10, -11, 0, -1, 5, -14, 14, -5, 1]]
